#! /usr/bin/env python
## ------------------------------------------------------------------------
## Copyright (C) 2018 Carnegie Mellon University. All Rights Reserved.
## ------------------------------------------------------------------------
## See license information in LICENSE-OPENSOURCE.txt
## ------------------------------------------------------------------------

import argparse
import os
import pyfixbuf
import pyfixbuf.cert
import shared_utils
from sets import Set


INDENT1 = '  '
INDENT2 = '    '


class Args:
    pass


class Template:
    def __init__(self, tid, fields):
        self.tid = tid
        self.fields = fields

    @classmethod
    def from_pyfixbuf(cls, tmpl, tmpl_id):
        fields = []
        for i in range(tmpl.__len__()):
            curr_spec = tmpl.__getitem__(i)
            fields.append(curr_spec.name)
        return cls(tmpl_id, fields)

    def print_template(self):
        global INDENT1
        global INDENT2
        print "%s%x" % (INDENT1, self.tid)
        for field in self.fields:
            if field != "basicList":         # don't print basicLists again
                print "%s%s" % (INDENT2, field)


def get_all_basiclist_fields(record, tmpl, infomodel):
    bl_fields = []
    for i in range(len(record)):
        elem_type = infomodel.get_element_type(tmpl[i].name)
        if elem_type == pyfixbuf.DataType.BASIC_LIST:
            bl_fields.append("basicList: " + record[i].element.name)
    return bl_fields


def get_stml_templates(stml, infomodel):
    global seen
    global templates
    for record in stml.iter_records():
        tmpl_id = record.template.template_id
        if tmpl_id not in seen:
            seen.add(tmpl_id)
            curr_template = Template.from_pyfixbuf(record.template, tmpl_id)
            if "basicList" in record:
                bl_fields = get_all_basiclist_fields(record, record.template, infomodel)
                curr_template.fields += bl_fields
            templates.append(curr_template)


def print_fields_tmpls(in_files):
    global INDENT1
    global INDENT2
    global seen
    global templates

    # Initialize lists of top-level fields and templates
    top_level_fields = []
    templates = []
    seen = Set([])

    # Set up collection capabilities
    infomodel = pyfixbuf.InfoModel()
    pyfixbuf.cert.add_elements_to_model(infomodel)

    for data in shared_utils.process_files(in_files, infomodel):
        for key, value in data.as_dict().items():
            elem_type = infomodel.get_element_type(key)
            if (key not in top_level_fields) and (elem_type < 20):
                top_level_fields.append(key)
            elif elem_type == pyfixbuf.DataType.SUB_TMPL_MULTI_LIST:
                get_stml_templates(value, infomodel)

    print "Top level fields:"
    for field in top_level_fields:
        print INDENT1 + field

    print "DPI Templates and Fields:"
    for tmpl in templates:
        tmpl.print_template()


if __name__ == '__main__':
    shared_utils.print_warning_message()

    in_files = []
    args = Args()

    # Create command-line argument parser and add arguments
    parser = argparse.ArgumentParser(prog='IPFinder.py',
                                     description='descripton: Process some IPFIX file(s)')
    parser.add_argument('--input', '--in', nargs=1, required=True,
                        help='input file or directory (no *)')

    # Add all command-line arguments to instance of class Args
    parser.parse_args(namespace=args)

    # handle input directory vs single input file
    if os.path.isdir(args.input[0]):
        for filename in os.listdir(args.input[0]):
            in_files.append(args.input[0] + '/' + filename)
    else:
        in_files.append(args.input[0])

    print_fields_tmpls(in_files)
