#! /usr/bin/env python
## ------------------------------------------------------------------------
## sample_exporter_collector_blind.py
##
## sample IPFIX collector & exporter (aka mediator) using pyfixbuf.
## created to be used with the sample_exporter.py script.
## Processes the ipfix file created by sample_exporter.py and writes
## to a text file.  This is different from sample_exporter_collector.py
## in that it has no idea what to expect.
## ------------------------------------------------------------------------
## Copyright (C) 2013-2018 Carnegie Mellon University. All Rights Reserved.
## ------------------------------------------------------------------------
## Authors: Emily Sarneso <ecoff@cert.org>
## ------------------------------------------------------------------------
## See license information in LICENSE-OPENSOURCE.txt

import os
import sys
import pyfixbuf
import pyfixbuf.cert

# Test that the argument number is correct
if (len(sys.argv) < 2 or sys.argv[1] in ['-h', '--help']):
    print "Must supply an IPFIX file to read and (optional) text file "
    "to write to."
    sys.exit()


#create the information model with the standard IPFIX elements
infomodel = pyfixbuf.InfoModel()

# add YAF's IPFIX elements
pyfixbuf.cert.add_elements_to_model(infomodel)

# create the collector
collector = pyfixbuf.Collector()

# create the IPFIX file to read from
collector.init_file(sys.argv[1])

# create the session
session = pyfixbuf.Session(infomodel)

# create the buffer for the collector, set auto to True to
# automatically generate templates
buf = pyfixbuf.Buffer(auto=True)

# make the buffer an import buffer
buf.init_collection(session, collector)

# set auto insert on buffer in case we receive any Info Element Options Recs
buf.auto_insert()

if (len(sys.argv) > 2):
    outFile = open(sys.argv[2], "w")
else:
    outFile = sys.stdout

flowcount = 0

dontprintlist=["subTemplateMultiList", "subTemplateList", "paddingOctets"]

for data in buf:

    for field in data.iterfields():
        if (field.name not in dontprintlist):
            outFile.write(field.name + ": " + str(field.value) + "\n")

    flowcount += 1
    # get data out of STML
    if "subTemplateMultiList" in data:
        stml = data["subTemplateMultiList"]
        subs = 0
        for entry in stml:
            print "--- STML %d ---" % subs
            recs = 0
            for record in entry:
                x = 0
                print "-- ENTRY %d --" % recs
                for elem in record:
                    if type(elem) is pyfixbuf.STL:
                        for item in elem:
                            for field in item.iterfields():
                                outFile.write(field.name + ": " +
                                              str(field.value) + "\n")
                        elem.clear()

                    elif type(elem) is pyfixbuf.BL:
                        count = 0
                        for item in elem:
                            outFile.write("%s %d is %s\n" %
                                          (elem.element.name, count, item))
                            count += 1
                        elem.clear()
                    else:
                        outFile.write("Item %d: %s\n" % (x, str(elem)))
                    x += 1
                recs += 1
            subs += 1

        stml.clear()
    outFile.write("------------------------------\n")

sys.stdout.write("Processed " + str(flowcount) + " flows \n")
